# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `RawPixelsStore.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_ModelF_ice
import omero_Collections_ice
import omero_ROMIO_ice
import omero_api_PyramidService_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Included module Glacier2
_M_Glacier2 = Ice.openModule('Glacier2')

# Included module omero.romio
_M_omero.romio = Ice.openModule('omero.romio')

# Included module omero.grid
_M_omero.grid = Ice.openModule('omero.grid')

# Start of module omero
__name__ = 'omero'

# Start of module omero.api
__name__ = 'omero.api'

if 'RawPixelsStore' not in _M_omero.api.__dict__:
    _M_omero.api.RawPixelsStore = Ice.createTempClass()
    class RawPixelsStore(_M_omero.api.PyramidService):
        """
        Binary data provider. Initialized with the ID of a
        omero.model.Pixels instance, this service can provide
        various slices, stacks, regions of the 5-dimensional (X-Y planes with
        multiple Z-sections and Channels over Time). The byte array returned
        by the getter methods and passed to the setter methods can and will
        be interpreted according to results of {@code getByteWidth},
        {@code isFloat}, and {@code isSigned}.
        Read-only caveat:
        Mutating methods (set*) are only available during the first access.
        Once the Pixels data has been successfully saved (via the save or
        close methods on this interface), then the data should be treated
        read-only. If Pixels data writing fails and the service is
        inadvertently closed, delete the Pixels object, and create a new
        one. Any partially written data will be removed.
        """
        def __init__(self):
            if Ice.getType(self) == _M_omero.api.RawPixelsStore:
                raise RuntimeError('omero.api.RawPixelsStore is an abstract class')

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::api::PyramidService', '::omero::api::RawPixelsStore', '::omero::api::ServiceInterface', '::omero::api::StatefulServiceInterface')

        def ice_id(self, current=None):
            return '::omero::api::RawPixelsStore'

        def ice_staticId():
            return '::omero::api::RawPixelsStore'
        ice_staticId = staticmethod(ice_staticId)

        def setPixelsId_async(self, _cb, pixelsId, bypassOriginalFile, current=None):
            """
            Initializes the stateful service for a given Pixels set.
            Arguments:
            _cb -- The asynchronous callback object.
            pixelsId -- Pixels set identifier.
            bypassOriginalFile -- Whether or not to bypass checking for an original file to back the pixel buffer used by this service. If requests are predominantly write-only or involve the population of a brand new pixel buffer using true here is a safe optimization otherwise false is expected. See Read-only caveat under RawPixelsStore
            current -- The Current object for the invocation.
            """
            pass

        def getPixelsId_async(self, _cb, current=None):
            """
            Returns the current Pixels set identifier.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def getPixelsPath_async(self, _cb, current=None):
            """
            Returns the current Pixels path.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def prepare_async(self, _cb, pixelsIds, current=None):
            """
            Prepares the stateful service with a cache of loaded Pixels
            objects.
            This method is designed to combat query overhead, where
            many sets of Pixels are to be read from or written to, by
            loading all the Pixels sets at once. Multiple calls will
            result in the existing cache being overwritten.
            Arguments:
            _cb -- The asynchronous callback object.
            pixelsIds -- Pixels IDs to cache.
            current -- The Current object for the invocation.
            """
            pass

        def getPlaneSize_async(self, _cb, current=None):
            """
            Retrieves the in memory size of a 2D image plane in this
            pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def getRowSize_async(self, _cb, current=None):
            """
            Retrieves the in memory size of a row or scanline of pixels
            in this pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def getStackSize_async(self, _cb, current=None):
            """
            Retrieves the in memory size of the entire number of
            optical sections for a single wavelength or channel
            at a particular timepoint in this pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def getTimepointSize_async(self, _cb, current=None):
            """
            Retrieves the in memory size of the entire number of
            optical sections for all wavelengths or channels at
            a particular timepoint in this pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def getTotalSize_async(self, _cb, current=None):
            """
            Retrieves the in memory size of the entire pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def getRowOffset_async(self, _cb, y, z, c, t, current=None):
            """
            Retrieves the offset for a particular row or scanline in
            this pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            y -- offset across the Y-axis of the pixel buffer.
            z -- offset across the Z-axis of the pixel buffer.
            c -- offset across the C-axis of the pixel buffer.
            t -- offset across the T-axis of the pixel buffer.
            current -- The Current object for the invocation.
            """
            pass

        def getPlaneOffset_async(self, _cb, z, c, t, current=None):
            """
            Retrieves the offset for a particular 2D image plane in this pixel
            store.
            Arguments:
            _cb -- The asynchronous callback object.
            z -- offset across the Z-axis of the pixel buffer.
            c -- offset across the C-axis of the pixel buffer.
            t -- offset across the T-axis of the pixel buffer.
            current -- The Current object for the invocation.
            """
            pass

        def getStackOffset_async(self, _cb, c, t, current=None):
            """
            Retrieves the offset for the entire number of optical
            sections for a single wavelength or channel at a
            particular timepoint in this pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            c -- offset across the C-axis of the pixel buffer.
            t -- offset across the T-axis of the pixel buffer.
            current -- The Current object for the invocation.
            """
            pass

        def getTimepointOffset_async(self, _cb, t, current=None):
            """
            Retrieves the in memory size of the entire number of
            optical sections for all wavelengths or channels at
            a particular timepoint in this pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            t -- offset across the T-axis of the pixel buffer.
            current -- The Current object for the invocation.
            """
            pass

        def getTile_async(self, _cb, z, c, t, x, y, w, h, current=None):
            """
            Retrieves a tile from this pixel buffer.
            Arguments:
            _cb -- The asynchronous callback object.
            z -- offset across the Z-axis of the pixel buffer.
            c -- offset across the C-axis of the pixel buffer.
            t -- offset across the T-axis of the pixel buffer.
            x -- Top left corner of the tile, X offset.
            y -- Top left corner of the tile, Y offset.
            w -- Width of the tile.
            h -- Height of the tile.
            current -- The Current object for the invocation.
            """
            pass

        def getHypercube_async(self, _cb, offset, size, step, current=None):
            """
            Retrieves a n-dimensional block from this pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            offset -- offset for each dimension within pixel store.
            size -- of each dimension (dependent on dimension).
            step -- needed of each dimension (dependent on dimension).
            current -- The Current object for the invocation.
            """
            pass

        def getRegion_async(self, _cb, size, offset, current=None):
            """
            Retrieves a region from this pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            size -- byte width of the region to retrieve.
            offset -- offset within the pixel store.
            current -- The Current object for the invocation.
            """
            pass

        def getRow_async(self, _cb, y, z, c, t, current=None):
            """
            Retrieves a particular row or scanline from this pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            y -- offset across the Y-axis of the pixel store.
            z -- offset across the Z-axis of the pixel store.
            c -- offset across the C-axis of the pixel store.
            t -- offset across the T-axis of the pixel store.
            current -- The Current object for the invocation.
            """
            pass

        def getCol_async(self, _cb, x, z, c, t, current=None):
            """
            Retrieves a particular column from this pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            x -- offset across the X-axis of the pixel store.
            z -- offset across the Z-axis of the pixel store.
            c -- offset across the C-axis of the pixel store.
            t -- offset across the T-axis of the pixel store.
            current -- The Current object for the invocation.
            """
            pass

        def getPlane_async(self, _cb, z, c, t, current=None):
            """
            Retrieves a particular 2D image plane from this pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            z -- offset across the Z-axis of the pixel store.
            c -- offset across the C-axis of the pixel store.
            t -- offset across the T-axis of the pixel store.
            current -- The Current object for the invocation.
            """
            pass

        def getPlaneRegion_async(self, _cb, z, c, t, size, offset, current=None):
            """
            Retrieves a region from a given plane from this pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            z -- offset across the Z-axis of the pixel store.
            c -- offset across the C-axis of the pixel store.
            t -- offset across the T-axis of the pixel store.
            size -- the number of pixels to retrieve.
            offset -- the offset at which to retrieve size pixels.
            current -- The Current object for the invocation.
            """
            pass

        def getStack_async(self, _cb, c, t, current=None):
            """
            Retrieves the the entire number of optical sections for a single
            wavelength or channel at a particular timepoint in this pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            c -- offset across the C-axis of the pixel store.
            t -- offset across the T-axis of the pixel store.
            current -- The Current object for the invocation.
            """
            pass

        def getTimepoint_async(self, _cb, t, current=None):
            """
            Retrieves the entire number of optical sections for all
            wavelengths or channels at a particular timepoint in this pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            t -- offset across the T-axis of the pixel store.
            current -- The Current object for the invocation.
            """
            pass

        def setTile_async(self, _cb, buf, z, c, t, x, y, w, h, current=None):
            """
            Sets a tile in this pixel buffer.
            Arguments:
            _cb -- The asynchronous callback object.
            buf -- A byte array of the data.
            z -- offset across the Z-axis of the pixel buffer.
            c -- offset across the C-axis of the pixel buffer.
            t -- offset across the T-axis of the pixel buffer.
            x -- Top left corner of the tile, X offset.
            y -- Top left corner of the tile, Y offset.
            w -- Width of the tile.
            h -- Height of the tile.
            current -- The Current object for the invocation.
            Throws:
            BufferOverflowException -- if an attempt is made to write off the end of the file. See Read-only caveat under RawPixelsStore
            IOException -- if there is a problem writing to the pixel buffer.
            """
            pass

        def setRegion_async(self, _cb, size, offset, buf, current=None):
            """
            Sets a region in this pixel buffer.
            Arguments:
            _cb -- The asynchronous callback object.
            size -- byte width of the region to set.
            offset -- offset within the pixel buffer.
            buf -- a byte array of the data. See Read-only caveat under RawPixelsStore
            current -- The Current object for the invocation.
            """
            pass

        def setRow_async(self, _cb, buf, y, z, c, t, current=None):
            """
            Sets a particular row or scanline in this pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            buf -- a byte array of the data comprising this row or scanline.
            y -- offset across the Y-axis of the pixel store.
            z -- offset across the Z-axis of the pixel store.
            c -- offset across the C-axis of the pixel store.
            t -- offset across the T-axis of the pixel store. See Read-only caveat under RawPixelsStore
            current -- The Current object for the invocation.
            """
            pass

        def setPlane_async(self, _cb, buf, z, c, t, current=None):
            """
            Sets a particular 2D image plane in this pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            buf -- a byte array of the data comprising this 2D image plane.
            z -- offset across the Z-axis of the pixel store.
            c -- offset across the C-axis of the pixel store.
            t -- offset across the T-axis of the pixel store. See Read-only caveat under RawPixelsStore
            current -- The Current object for the invocation.
            """
            pass

        def setStack_async(self, _cb, buf, z, c, t, current=None):
            """
            Sets the entire number of optical sections for a single
            wavelength or channel at a particular timepoint in this pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            buf -- a byte array of the data comprising this stack.
            z -- 
            c -- offset across the C-axis of the pixel store.
            t -- offset across the T-axis of the pixel store. See Read-only caveat under RawPixelsStore
            current -- The Current object for the invocation.
            """
            pass

        def setTimepoint_async(self, _cb, buf, t, current=None):
            """
            Sets the entire number of optical sections for all
            wavelengths or channels at a particular timepoint in this pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            buf -- a byte array of the data comprising this timepoint.
            t -- offset across the T-axis of the pixel buffer. See Read-only caveat under RawPixelsStore
            current -- The Current object for the invocation.
            """
            pass

        def getHistogram_async(self, _cb, channels, binCount, globalRange, plane, current=None):
            """
            Retrieves the histogram data for the specified plane and channels. This method can currently only handle non-pyramid images.
            Arguments:
            _cb -- The asynchronous callback object.
            channels -- the channels to generate the histogram data for
            binCount -- the number of the histogram bins (optional, default: 256)
            globalRange -- use the global minimum/maximum to determine the histogram range, otherwise use plane minimum/maximum value
            plane -- the plane (optional, default: whole region of first z/t plane)
            current -- The Current object for the invocation.
            """
            pass

        def findMinMax_async(self, _cb, channels, current=None):
            """
            Find the minimum and maximum pixel values for the specified channels by iterating over a full plane.
            In case of multi-z/t images only the 'middle' plane with index maxZ/2, respectively maxT/2 is taken into account.
            Note: This method can currently only handle non-pyramid images, otherwise an empty map will be returned.
            Arguments:
            _cb -- The asynchronous callback object.
            channels -- the channels
            current -- The Current object for the invocation.
            """
            pass

        def getByteWidth_async(self, _cb, current=None):
            """
            Returns the byte width for the pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def isSigned_async(self, _cb, current=None):
            """
            Returns whether or not the pixel store has signed pixels.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def isFloat_async(self, _cb, current=None):
            """
            Returns whether or not the pixel buffer has floating point pixels.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def calculateMessageDigest_async(self, _cb, current=None):
            """
            Calculates a SHA-1 message digest for the entire pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def save_async(self, _cb, current=None):
            """
            Save the current state of the pixels, updating the SHA1. This should
            only be called AFTER all data is successfully set. Future invocations
            of set methods may be disallowed. This read-only status will allow
            background processing (generation of thumbnails, compression, etc.)
            to begin. More information under RawPixelsStore.
            A null instance will be returned if no save was performed.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.api._t_RawPixelsStore)

        __repr__ = __str__

    _M_omero.api.RawPixelsStorePrx = Ice.createTempClass()
    class RawPixelsStorePrx(_M_omero.api.PyramidServicePrx):

        """
        Initializes the stateful service for a given Pixels set.
        Arguments:
        pixelsId -- Pixels set identifier.
        bypassOriginalFile -- Whether or not to bypass checking for an original file to back the pixel buffer used by this service. If requests are predominantly write-only or involve the population of a brand new pixel buffer using true here is a safe optimization otherwise false is expected. See Read-only caveat under RawPixelsStore
        _ctx -- The request context for the invocation.
        """
        def setPixelsId(self, pixelsId, bypassOriginalFile, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_setPixelsId.invoke(self, ((pixelsId, bypassOriginalFile), _ctx))

        """
        Initializes the stateful service for a given Pixels set.
        Arguments:
        pixelsId -- Pixels set identifier.
        bypassOriginalFile -- Whether or not to bypass checking for an original file to back the pixel buffer used by this service. If requests are predominantly write-only or involve the population of a brand new pixel buffer using true here is a safe optimization otherwise false is expected. See Read-only caveat under RawPixelsStore
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setPixelsId(self, pixelsId, bypassOriginalFile, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_setPixelsId.begin(self, ((pixelsId, bypassOriginalFile), _response, _ex, _sent, _ctx))

        """
        Initializes the stateful service for a given Pixels set.
        Arguments:
        pixelsId -- Pixels set identifier.
        bypassOriginalFile -- Whether or not to bypass checking for an original file to back the pixel buffer used by this service. If requests are predominantly write-only or involve the population of a brand new pixel buffer using true here is a safe optimization otherwise false is expected. See Read-only caveat under RawPixelsStore
        """
        def end_setPixelsId(self, _r):
            return _M_omero.api.RawPixelsStore._op_setPixelsId.end(self, _r)

        """
        Returns the current Pixels set identifier.
        Arguments:
        _ctx -- The request context for the invocation.
        Returns: See above.
        """
        def getPixelsId(self, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getPixelsId.invoke(self, ((), _ctx))

        """
        Returns the current Pixels set identifier.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getPixelsId(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getPixelsId.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Returns the current Pixels set identifier.
        Arguments:
        Returns: See above.
        """
        def end_getPixelsId(self, _r):
            return _M_omero.api.RawPixelsStore._op_getPixelsId.end(self, _r)

        """
        Returns the current Pixels path.
        Arguments:
        _ctx -- The request context for the invocation.
        Returns: See above.
        """
        def getPixelsPath(self, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getPixelsPath.invoke(self, ((), _ctx))

        """
        Returns the current Pixels path.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getPixelsPath(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getPixelsPath.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Returns the current Pixels path.
        Arguments:
        Returns: See above.
        """
        def end_getPixelsPath(self, _r):
            return _M_omero.api.RawPixelsStore._op_getPixelsPath.end(self, _r)

        """
        Prepares the stateful service with a cache of loaded Pixels
        objects.
        This method is designed to combat query overhead, where
        many sets of Pixels are to be read from or written to, by
        loading all the Pixels sets at once. Multiple calls will
        result in the existing cache being overwritten.
        Arguments:
        pixelsIds -- Pixels IDs to cache.
        _ctx -- The request context for the invocation.
        """
        def prepare(self, pixelsIds, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_prepare.invoke(self, ((pixelsIds, ), _ctx))

        """
        Prepares the stateful service with a cache of loaded Pixels
        objects.
        This method is designed to combat query overhead, where
        many sets of Pixels are to be read from or written to, by
        loading all the Pixels sets at once. Multiple calls will
        result in the existing cache being overwritten.
        Arguments:
        pixelsIds -- Pixels IDs to cache.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_prepare(self, pixelsIds, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_prepare.begin(self, ((pixelsIds, ), _response, _ex, _sent, _ctx))

        """
        Prepares the stateful service with a cache of loaded Pixels
        objects.
        This method is designed to combat query overhead, where
        many sets of Pixels are to be read from or written to, by
        loading all the Pixels sets at once. Multiple calls will
        result in the existing cache being overwritten.
        Arguments:
        pixelsIds -- Pixels IDs to cache.
        """
        def end_prepare(self, _r):
            return _M_omero.api.RawPixelsStore._op_prepare.end(self, _r)

        """
        Retrieves the in memory size of a 2D image plane in this
        pixel store.
        Arguments:
        _ctx -- The request context for the invocation.
        Returns: 2D image plane size in bytes sizeX*sizeY*byteWidth.
        """
        def getPlaneSize(self, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getPlaneSize.invoke(self, ((), _ctx))

        """
        Retrieves the in memory size of a 2D image plane in this
        pixel store.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getPlaneSize(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getPlaneSize.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Retrieves the in memory size of a 2D image plane in this
        pixel store.
        Arguments:
        Returns: 2D image plane size in bytes sizeX*sizeY*byteWidth.
        """
        def end_getPlaneSize(self, _r):
            return _M_omero.api.RawPixelsStore._op_getPlaneSize.end(self, _r)

        """
        Retrieves the in memory size of a row or scanline of pixels
        in this pixel store.
        Arguments:
        _ctx -- The request context for the invocation.
        Returns: row or scanline size in bytes sizeX*byteWidth
        """
        def getRowSize(self, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getRowSize.invoke(self, ((), _ctx))

        """
        Retrieves the in memory size of a row or scanline of pixels
        in this pixel store.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getRowSize(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getRowSize.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Retrieves the in memory size of a row or scanline of pixels
        in this pixel store.
        Arguments:
        Returns: row or scanline size in bytes sizeX*byteWidth
        """
        def end_getRowSize(self, _r):
            return _M_omero.api.RawPixelsStore._op_getRowSize.end(self, _r)

        """
        Retrieves the in memory size of the entire number of
        optical sections for a single wavelength or channel
        at a particular timepoint in this pixel store.
        Arguments:
        _ctx -- The request context for the invocation.
        Returns: stack size in bytes sizeX*sizeY*byteWidth.
        """
        def getStackSize(self, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getStackSize.invoke(self, ((), _ctx))

        """
        Retrieves the in memory size of the entire number of
        optical sections for a single wavelength or channel
        at a particular timepoint in this pixel store.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getStackSize(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getStackSize.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Retrieves the in memory size of the entire number of
        optical sections for a single wavelength or channel
        at a particular timepoint in this pixel store.
        Arguments:
        Returns: stack size in bytes sizeX*sizeY*byteWidth.
        """
        def end_getStackSize(self, _r):
            return _M_omero.api.RawPixelsStore._op_getStackSize.end(self, _r)

        """
        Retrieves the in memory size of the entire number of
        optical sections for all wavelengths or channels at
        a particular timepoint in this pixel store.
        Arguments:
        _ctx -- The request context for the invocation.
        Returns: timepoint size in bytes sizeX*sizeY*sizeZ*sizeC*byteWidth.
        """
        def getTimepointSize(self, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getTimepointSize.invoke(self, ((), _ctx))

        """
        Retrieves the in memory size of the entire number of
        optical sections for all wavelengths or channels at
        a particular timepoint in this pixel store.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getTimepointSize(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getTimepointSize.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Retrieves the in memory size of the entire number of
        optical sections for all wavelengths or channels at
        a particular timepoint in this pixel store.
        Arguments:
        Returns: timepoint size in bytes sizeX*sizeY*sizeZ*sizeC*byteWidth.
        """
        def end_getTimepointSize(self, _r):
            return _M_omero.api.RawPixelsStore._op_getTimepointSize.end(self, _r)

        """
        Retrieves the in memory size of the entire pixel store.
        Arguments:
        _ctx -- The request context for the invocation.
        Returns: total size of the pixel size in bytes sizeX*sizeY*sizeZ*sizeC*sizeT*byteWidth.
        """
        def getTotalSize(self, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getTotalSize.invoke(self, ((), _ctx))

        """
        Retrieves the in memory size of the entire pixel store.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getTotalSize(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getTotalSize.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Retrieves the in memory size of the entire pixel store.
        Arguments:
        Returns: total size of the pixel size in bytes sizeX*sizeY*sizeZ*sizeC*sizeT*byteWidth.
        """
        def end_getTotalSize(self, _r):
            return _M_omero.api.RawPixelsStore._op_getTotalSize.end(self, _r)

        """
        Retrieves the offset for a particular row or scanline in
        this pixel store.
        Arguments:
        y -- offset across the Y-axis of the pixel buffer.
        z -- offset across the Z-axis of the pixel buffer.
        c -- offset across the C-axis of the pixel buffer.
        t -- offset across the T-axis of the pixel buffer.
        _ctx -- The request context for the invocation.
        Returns: offset of the row or scanline.
        """
        def getRowOffset(self, y, z, c, t, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getRowOffset.invoke(self, ((y, z, c, t), _ctx))

        """
        Retrieves the offset for a particular row or scanline in
        this pixel store.
        Arguments:
        y -- offset across the Y-axis of the pixel buffer.
        z -- offset across the Z-axis of the pixel buffer.
        c -- offset across the C-axis of the pixel buffer.
        t -- offset across the T-axis of the pixel buffer.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getRowOffset(self, y, z, c, t, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getRowOffset.begin(self, ((y, z, c, t), _response, _ex, _sent, _ctx))

        """
        Retrieves the offset for a particular row or scanline in
        this pixel store.
        Arguments:
        y -- offset across the Y-axis of the pixel buffer.
        z -- offset across the Z-axis of the pixel buffer.
        c -- offset across the C-axis of the pixel buffer.
        t -- offset across the T-axis of the pixel buffer.
        Returns: offset of the row or scanline.
        """
        def end_getRowOffset(self, _r):
            return _M_omero.api.RawPixelsStore._op_getRowOffset.end(self, _r)

        """
        Retrieves the offset for a particular 2D image plane in this pixel
        store.
        Arguments:
        z -- offset across the Z-axis of the pixel buffer.
        c -- offset across the C-axis of the pixel buffer.
        t -- offset across the T-axis of the pixel buffer.
        _ctx -- The request context for the invocation.
        Returns: offset of the 2D image plane.
        """
        def getPlaneOffset(self, z, c, t, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getPlaneOffset.invoke(self, ((z, c, t), _ctx))

        """
        Retrieves the offset for a particular 2D image plane in this pixel
        store.
        Arguments:
        z -- offset across the Z-axis of the pixel buffer.
        c -- offset across the C-axis of the pixel buffer.
        t -- offset across the T-axis of the pixel buffer.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getPlaneOffset(self, z, c, t, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getPlaneOffset.begin(self, ((z, c, t), _response, _ex, _sent, _ctx))

        """
        Retrieves the offset for a particular 2D image plane in this pixel
        store.
        Arguments:
        z -- offset across the Z-axis of the pixel buffer.
        c -- offset across the C-axis of the pixel buffer.
        t -- offset across the T-axis of the pixel buffer.
        Returns: offset of the 2D image plane.
        """
        def end_getPlaneOffset(self, _r):
            return _M_omero.api.RawPixelsStore._op_getPlaneOffset.end(self, _r)

        """
        Retrieves the offset for the entire number of optical
        sections for a single wavelength or channel at a
        particular timepoint in this pixel store.
        Arguments:
        c -- offset across the C-axis of the pixel buffer.
        t -- offset across the T-axis of the pixel buffer.
        _ctx -- The request context for the invocation.
        Returns: offset of the stack.
        """
        def getStackOffset(self, c, t, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getStackOffset.invoke(self, ((c, t), _ctx))

        """
        Retrieves the offset for the entire number of optical
        sections for a single wavelength or channel at a
        particular timepoint in this pixel store.
        Arguments:
        c -- offset across the C-axis of the pixel buffer.
        t -- offset across the T-axis of the pixel buffer.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getStackOffset(self, c, t, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getStackOffset.begin(self, ((c, t), _response, _ex, _sent, _ctx))

        """
        Retrieves the offset for the entire number of optical
        sections for a single wavelength or channel at a
        particular timepoint in this pixel store.
        Arguments:
        c -- offset across the C-axis of the pixel buffer.
        t -- offset across the T-axis of the pixel buffer.
        Returns: offset of the stack.
        """
        def end_getStackOffset(self, _r):
            return _M_omero.api.RawPixelsStore._op_getStackOffset.end(self, _r)

        """
        Retrieves the in memory size of the entire number of
        optical sections for all wavelengths or channels at
        a particular timepoint in this pixel store.
        Arguments:
        t -- offset across the T-axis of the pixel buffer.
        _ctx -- The request context for the invocation.
        Returns: offset of the timepoint.
        """
        def getTimepointOffset(self, t, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getTimepointOffset.invoke(self, ((t, ), _ctx))

        """
        Retrieves the in memory size of the entire number of
        optical sections for all wavelengths or channels at
        a particular timepoint in this pixel store.
        Arguments:
        t -- offset across the T-axis of the pixel buffer.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getTimepointOffset(self, t, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getTimepointOffset.begin(self, ((t, ), _response, _ex, _sent, _ctx))

        """
        Retrieves the in memory size of the entire number of
        optical sections for all wavelengths or channels at
        a particular timepoint in this pixel store.
        Arguments:
        t -- offset across the T-axis of the pixel buffer.
        Returns: offset of the timepoint.
        """
        def end_getTimepointOffset(self, _r):
            return _M_omero.api.RawPixelsStore._op_getTimepointOffset.end(self, _r)

        """
        Retrieves a tile from this pixel buffer.
        Arguments:
        z -- offset across the Z-axis of the pixel buffer.
        c -- offset across the C-axis of the pixel buffer.
        t -- offset across the T-axis of the pixel buffer.
        x -- Top left corner of the tile, X offset.
        y -- Top left corner of the tile, Y offset.
        w -- Width of the tile.
        h -- Height of the tile.
        _ctx -- The request context for the invocation.
        Returns: buffer containing the data.
        """
        def getTile(self, z, c, t, x, y, w, h, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getTile.invoke(self, ((z, c, t, x, y, w, h), _ctx))

        """
        Retrieves a tile from this pixel buffer.
        Arguments:
        z -- offset across the Z-axis of the pixel buffer.
        c -- offset across the C-axis of the pixel buffer.
        t -- offset across the T-axis of the pixel buffer.
        x -- Top left corner of the tile, X offset.
        y -- Top left corner of the tile, Y offset.
        w -- Width of the tile.
        h -- Height of the tile.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getTile(self, z, c, t, x, y, w, h, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getTile.begin(self, ((z, c, t, x, y, w, h), _response, _ex, _sent, _ctx))

        """
        Retrieves a tile from this pixel buffer.
        Arguments:
        z -- offset across the Z-axis of the pixel buffer.
        c -- offset across the C-axis of the pixel buffer.
        t -- offset across the T-axis of the pixel buffer.
        x -- Top left corner of the tile, X offset.
        y -- Top left corner of the tile, Y offset.
        w -- Width of the tile.
        h -- Height of the tile.
        Returns: buffer containing the data.
        """
        def end_getTile(self, _r):
            return _M_omero.api.RawPixelsStore._op_getTile.end(self, _r)

        """
        Retrieves a n-dimensional block from this pixel store.
        Arguments:
        offset -- offset for each dimension within pixel store.
        size -- of each dimension (dependent on dimension).
        step -- needed of each dimension (dependent on dimension).
        _ctx -- The request context for the invocation.
        Returns: buffer containing the data.
        """
        def getHypercube(self, offset, size, step, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getHypercube.invoke(self, ((offset, size, step), _ctx))

        """
        Retrieves a n-dimensional block from this pixel store.
        Arguments:
        offset -- offset for each dimension within pixel store.
        size -- of each dimension (dependent on dimension).
        step -- needed of each dimension (dependent on dimension).
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getHypercube(self, offset, size, step, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getHypercube.begin(self, ((offset, size, step), _response, _ex, _sent, _ctx))

        """
        Retrieves a n-dimensional block from this pixel store.
        Arguments:
        offset -- offset for each dimension within pixel store.
        size -- of each dimension (dependent on dimension).
        step -- needed of each dimension (dependent on dimension).
        Returns: buffer containing the data.
        """
        def end_getHypercube(self, _r):
            return _M_omero.api.RawPixelsStore._op_getHypercube.end(self, _r)

        """
        Retrieves a region from this pixel store.
        Arguments:
        size -- byte width of the region to retrieve.
        offset -- offset within the pixel store.
        _ctx -- The request context for the invocation.
        Returns: buffer containing the data.
        """
        def getRegion(self, size, offset, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getRegion.invoke(self, ((size, offset), _ctx))

        """
        Retrieves a region from this pixel store.
        Arguments:
        size -- byte width of the region to retrieve.
        offset -- offset within the pixel store.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getRegion(self, size, offset, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getRegion.begin(self, ((size, offset), _response, _ex, _sent, _ctx))

        """
        Retrieves a region from this pixel store.
        Arguments:
        size -- byte width of the region to retrieve.
        offset -- offset within the pixel store.
        Returns: buffer containing the data.
        """
        def end_getRegion(self, _r):
            return _M_omero.api.RawPixelsStore._op_getRegion.end(self, _r)

        """
        Retrieves a particular row or scanline from this pixel store.
        Arguments:
        y -- offset across the Y-axis of the pixel store.
        z -- offset across the Z-axis of the pixel store.
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store.
        _ctx -- The request context for the invocation.
        Returns: buffer containing the data which comprises this row or scanline.
        """
        def getRow(self, y, z, c, t, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getRow.invoke(self, ((y, z, c, t), _ctx))

        """
        Retrieves a particular row or scanline from this pixel store.
        Arguments:
        y -- offset across the Y-axis of the pixel store.
        z -- offset across the Z-axis of the pixel store.
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getRow(self, y, z, c, t, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getRow.begin(self, ((y, z, c, t), _response, _ex, _sent, _ctx))

        """
        Retrieves a particular row or scanline from this pixel store.
        Arguments:
        y -- offset across the Y-axis of the pixel store.
        z -- offset across the Z-axis of the pixel store.
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store.
        Returns: buffer containing the data which comprises this row or scanline.
        """
        def end_getRow(self, _r):
            return _M_omero.api.RawPixelsStore._op_getRow.end(self, _r)

        """
        Retrieves a particular column from this pixel store.
        Arguments:
        x -- offset across the X-axis of the pixel store.
        z -- offset across the Z-axis of the pixel store.
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store.
        _ctx -- The request context for the invocation.
        Returns: buffer containing the data which comprises this column.
        """
        def getCol(self, x, z, c, t, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getCol.invoke(self, ((x, z, c, t), _ctx))

        """
        Retrieves a particular column from this pixel store.
        Arguments:
        x -- offset across the X-axis of the pixel store.
        z -- offset across the Z-axis of the pixel store.
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getCol(self, x, z, c, t, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getCol.begin(self, ((x, z, c, t), _response, _ex, _sent, _ctx))

        """
        Retrieves a particular column from this pixel store.
        Arguments:
        x -- offset across the X-axis of the pixel store.
        z -- offset across the Z-axis of the pixel store.
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store.
        Returns: buffer containing the data which comprises this column.
        """
        def end_getCol(self, _r):
            return _M_omero.api.RawPixelsStore._op_getCol.end(self, _r)

        """
        Retrieves a particular 2D image plane from this pixel store.
        Arguments:
        z -- offset across the Z-axis of the pixel store.
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store.
        _ctx -- The request context for the invocation.
        Returns: buffer containing the data which comprises this 2D image plane.
        """
        def getPlane(self, z, c, t, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getPlane.invoke(self, ((z, c, t), _ctx))

        """
        Retrieves a particular 2D image plane from this pixel store.
        Arguments:
        z -- offset across the Z-axis of the pixel store.
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getPlane(self, z, c, t, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getPlane.begin(self, ((z, c, t), _response, _ex, _sent, _ctx))

        """
        Retrieves a particular 2D image plane from this pixel store.
        Arguments:
        z -- offset across the Z-axis of the pixel store.
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store.
        Returns: buffer containing the data which comprises this 2D image plane.
        """
        def end_getPlane(self, _r):
            return _M_omero.api.RawPixelsStore._op_getPlane.end(self, _r)

        """
        Retrieves a region from a given plane from this pixel store.
        Arguments:
        z -- offset across the Z-axis of the pixel store.
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store.
        size -- the number of pixels to retrieve.
        offset -- the offset at which to retrieve size pixels.
        _ctx -- The request context for the invocation.
        Returns: buffer containing the data which comprises the region of the given 2D image plane. It is guaranteed that this buffer will have been byte swapped.
        """
        def getPlaneRegion(self, z, c, t, size, offset, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getPlaneRegion.invoke(self, ((z, c, t, size, offset), _ctx))

        """
        Retrieves a region from a given plane from this pixel store.
        Arguments:
        z -- offset across the Z-axis of the pixel store.
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store.
        size -- the number of pixels to retrieve.
        offset -- the offset at which to retrieve size pixels.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getPlaneRegion(self, z, c, t, size, offset, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getPlaneRegion.begin(self, ((z, c, t, size, offset), _response, _ex, _sent, _ctx))

        """
        Retrieves a region from a given plane from this pixel store.
        Arguments:
        z -- offset across the Z-axis of the pixel store.
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store.
        size -- the number of pixels to retrieve.
        offset -- the offset at which to retrieve size pixels.
        Returns: buffer containing the data which comprises the region of the given 2D image plane. It is guaranteed that this buffer will have been byte swapped.
        """
        def end_getPlaneRegion(self, _r):
            return _M_omero.api.RawPixelsStore._op_getPlaneRegion.end(self, _r)

        """
        Retrieves the the entire number of optical sections for a single
        wavelength or channel at a particular timepoint in this pixel store.
        Arguments:
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store.
        _ctx -- The request context for the invocation.
        Returns: buffer containing the data which comprises this stack.
        """
        def getStack(self, c, t, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getStack.invoke(self, ((c, t), _ctx))

        """
        Retrieves the the entire number of optical sections for a single
        wavelength or channel at a particular timepoint in this pixel store.
        Arguments:
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getStack(self, c, t, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getStack.begin(self, ((c, t), _response, _ex, _sent, _ctx))

        """
        Retrieves the the entire number of optical sections for a single
        wavelength or channel at a particular timepoint in this pixel store.
        Arguments:
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store.
        Returns: buffer containing the data which comprises this stack.
        """
        def end_getStack(self, _r):
            return _M_omero.api.RawPixelsStore._op_getStack.end(self, _r)

        """
        Retrieves the entire number of optical sections for all
        wavelengths or channels at a particular timepoint in this pixel store.
        Arguments:
        t -- offset across the T-axis of the pixel store.
        _ctx -- The request context for the invocation.
        Returns: buffer containing the data which comprises this stack.
        """
        def getTimepoint(self, t, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getTimepoint.invoke(self, ((t, ), _ctx))

        """
        Retrieves the entire number of optical sections for all
        wavelengths or channels at a particular timepoint in this pixel store.
        Arguments:
        t -- offset across the T-axis of the pixel store.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getTimepoint(self, t, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getTimepoint.begin(self, ((t, ), _response, _ex, _sent, _ctx))

        """
        Retrieves the entire number of optical sections for all
        wavelengths or channels at a particular timepoint in this pixel store.
        Arguments:
        t -- offset across the T-axis of the pixel store.
        Returns: buffer containing the data which comprises this stack.
        """
        def end_getTimepoint(self, _r):
            return _M_omero.api.RawPixelsStore._op_getTimepoint.end(self, _r)

        """
        Sets a tile in this pixel buffer.
        Arguments:
        buf -- A byte array of the data.
        z -- offset across the Z-axis of the pixel buffer.
        c -- offset across the C-axis of the pixel buffer.
        t -- offset across the T-axis of the pixel buffer.
        x -- Top left corner of the tile, X offset.
        y -- Top left corner of the tile, Y offset.
        w -- Width of the tile.
        h -- Height of the tile.
        _ctx -- The request context for the invocation.
        Throws:
        BufferOverflowException -- if an attempt is made to write off the end of the file. See Read-only caveat under RawPixelsStore
        IOException -- if there is a problem writing to the pixel buffer.
        """
        def setTile(self, buf, z, c, t, x, y, w, h, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_setTile.invoke(self, ((buf, z, c, t, x, y, w, h), _ctx))

        """
        Sets a tile in this pixel buffer.
        Arguments:
        buf -- A byte array of the data.
        z -- offset across the Z-axis of the pixel buffer.
        c -- offset across the C-axis of the pixel buffer.
        t -- offset across the T-axis of the pixel buffer.
        x -- Top left corner of the tile, X offset.
        y -- Top left corner of the tile, Y offset.
        w -- Width of the tile.
        h -- Height of the tile.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setTile(self, buf, z, c, t, x, y, w, h, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_setTile.begin(self, ((buf, z, c, t, x, y, w, h), _response, _ex, _sent, _ctx))

        """
        Sets a tile in this pixel buffer.
        Arguments:
        buf -- A byte array of the data.
        z -- offset across the Z-axis of the pixel buffer.
        c -- offset across the C-axis of the pixel buffer.
        t -- offset across the T-axis of the pixel buffer.
        x -- Top left corner of the tile, X offset.
        y -- Top left corner of the tile, Y offset.
        w -- Width of the tile.
        h -- Height of the tile.
        Throws:
        BufferOverflowException -- if an attempt is made to write off the end of the file. See Read-only caveat under RawPixelsStore
        IOException -- if there is a problem writing to the pixel buffer.
        """
        def end_setTile(self, _r):
            return _M_omero.api.RawPixelsStore._op_setTile.end(self, _r)

        """
        Sets a region in this pixel buffer.
        Arguments:
        size -- byte width of the region to set.
        offset -- offset within the pixel buffer.
        buf -- a byte array of the data. See Read-only caveat under RawPixelsStore
        _ctx -- The request context for the invocation.
        """
        def setRegion(self, size, offset, buf, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_setRegion.invoke(self, ((size, offset, buf), _ctx))

        """
        Sets a region in this pixel buffer.
        Arguments:
        size -- byte width of the region to set.
        offset -- offset within the pixel buffer.
        buf -- a byte array of the data. See Read-only caveat under RawPixelsStore
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setRegion(self, size, offset, buf, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_setRegion.begin(self, ((size, offset, buf), _response, _ex, _sent, _ctx))

        """
        Sets a region in this pixel buffer.
        Arguments:
        size -- byte width of the region to set.
        offset -- offset within the pixel buffer.
        buf -- a byte array of the data. See Read-only caveat under RawPixelsStore
        """
        def end_setRegion(self, _r):
            return _M_omero.api.RawPixelsStore._op_setRegion.end(self, _r)

        """
        Sets a particular row or scanline in this pixel store.
        Arguments:
        buf -- a byte array of the data comprising this row or scanline.
        y -- offset across the Y-axis of the pixel store.
        z -- offset across the Z-axis of the pixel store.
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store. See Read-only caveat under RawPixelsStore
        _ctx -- The request context for the invocation.
        """
        def setRow(self, buf, y, z, c, t, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_setRow.invoke(self, ((buf, y, z, c, t), _ctx))

        """
        Sets a particular row or scanline in this pixel store.
        Arguments:
        buf -- a byte array of the data comprising this row or scanline.
        y -- offset across the Y-axis of the pixel store.
        z -- offset across the Z-axis of the pixel store.
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store. See Read-only caveat under RawPixelsStore
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setRow(self, buf, y, z, c, t, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_setRow.begin(self, ((buf, y, z, c, t), _response, _ex, _sent, _ctx))

        """
        Sets a particular row or scanline in this pixel store.
        Arguments:
        buf -- a byte array of the data comprising this row or scanline.
        y -- offset across the Y-axis of the pixel store.
        z -- offset across the Z-axis of the pixel store.
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store. See Read-only caveat under RawPixelsStore
        """
        def end_setRow(self, _r):
            return _M_omero.api.RawPixelsStore._op_setRow.end(self, _r)

        """
        Sets a particular 2D image plane in this pixel store.
        Arguments:
        buf -- a byte array of the data comprising this 2D image plane.
        z -- offset across the Z-axis of the pixel store.
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store. See Read-only caveat under RawPixelsStore
        _ctx -- The request context for the invocation.
        """
        def setPlane(self, buf, z, c, t, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_setPlane.invoke(self, ((buf, z, c, t), _ctx))

        """
        Sets a particular 2D image plane in this pixel store.
        Arguments:
        buf -- a byte array of the data comprising this 2D image plane.
        z -- offset across the Z-axis of the pixel store.
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store. See Read-only caveat under RawPixelsStore
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setPlane(self, buf, z, c, t, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_setPlane.begin(self, ((buf, z, c, t), _response, _ex, _sent, _ctx))

        """
        Sets a particular 2D image plane in this pixel store.
        Arguments:
        buf -- a byte array of the data comprising this 2D image plane.
        z -- offset across the Z-axis of the pixel store.
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store. See Read-only caveat under RawPixelsStore
        """
        def end_setPlane(self, _r):
            return _M_omero.api.RawPixelsStore._op_setPlane.end(self, _r)

        """
        Sets the entire number of optical sections for a single
        wavelength or channel at a particular timepoint in this pixel store.
        Arguments:
        buf -- a byte array of the data comprising this stack.
        z -- 
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store. See Read-only caveat under RawPixelsStore
        _ctx -- The request context for the invocation.
        """
        def setStack(self, buf, z, c, t, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_setStack.invoke(self, ((buf, z, c, t), _ctx))

        """
        Sets the entire number of optical sections for a single
        wavelength or channel at a particular timepoint in this pixel store.
        Arguments:
        buf -- a byte array of the data comprising this stack.
        z -- 
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store. See Read-only caveat under RawPixelsStore
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setStack(self, buf, z, c, t, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_setStack.begin(self, ((buf, z, c, t), _response, _ex, _sent, _ctx))

        """
        Sets the entire number of optical sections for a single
        wavelength or channel at a particular timepoint in this pixel store.
        Arguments:
        buf -- a byte array of the data comprising this stack.
        z -- 
        c -- offset across the C-axis of the pixel store.
        t -- offset across the T-axis of the pixel store. See Read-only caveat under RawPixelsStore
        """
        def end_setStack(self, _r):
            return _M_omero.api.RawPixelsStore._op_setStack.end(self, _r)

        """
        Sets the entire number of optical sections for all
        wavelengths or channels at a particular timepoint in this pixel store.
        Arguments:
        buf -- a byte array of the data comprising this timepoint.
        t -- offset across the T-axis of the pixel buffer. See Read-only caveat under RawPixelsStore
        _ctx -- The request context for the invocation.
        """
        def setTimepoint(self, buf, t, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_setTimepoint.invoke(self, ((buf, t), _ctx))

        """
        Sets the entire number of optical sections for all
        wavelengths or channels at a particular timepoint in this pixel store.
        Arguments:
        buf -- a byte array of the data comprising this timepoint.
        t -- offset across the T-axis of the pixel buffer. See Read-only caveat under RawPixelsStore
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setTimepoint(self, buf, t, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_setTimepoint.begin(self, ((buf, t), _response, _ex, _sent, _ctx))

        """
        Sets the entire number of optical sections for all
        wavelengths or channels at a particular timepoint in this pixel store.
        Arguments:
        buf -- a byte array of the data comprising this timepoint.
        t -- offset across the T-axis of the pixel buffer. See Read-only caveat under RawPixelsStore
        """
        def end_setTimepoint(self, _r):
            return _M_omero.api.RawPixelsStore._op_setTimepoint.end(self, _r)

        """
        Retrieves the histogram data for the specified plane and channels. This method can currently only handle non-pyramid images.
        Arguments:
        channels -- the channels to generate the histogram data for
        binCount -- the number of the histogram bins (optional, default: 256)
        globalRange -- use the global minimum/maximum to determine the histogram range, otherwise use plane minimum/maximum value
        plane -- the plane (optional, default: whole region of first z/t plane)
        _ctx -- The request context for the invocation.
        Returns: See above.
        """
        def getHistogram(self, channels, binCount, globalRange, plane, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getHistogram.invoke(self, ((channels, binCount, globalRange, plane), _ctx))

        """
        Retrieves the histogram data for the specified plane and channels. This method can currently only handle non-pyramid images.
        Arguments:
        channels -- the channels to generate the histogram data for
        binCount -- the number of the histogram bins (optional, default: 256)
        globalRange -- use the global minimum/maximum to determine the histogram range, otherwise use plane minimum/maximum value
        plane -- the plane (optional, default: whole region of first z/t plane)
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getHistogram(self, channels, binCount, globalRange, plane, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getHistogram.begin(self, ((channels, binCount, globalRange, plane), _response, _ex, _sent, _ctx))

        """
        Retrieves the histogram data for the specified plane and channels. This method can currently only handle non-pyramid images.
        Arguments:
        channels -- the channels to generate the histogram data for
        binCount -- the number of the histogram bins (optional, default: 256)
        globalRange -- use the global minimum/maximum to determine the histogram range, otherwise use plane minimum/maximum value
        plane -- the plane (optional, default: whole region of first z/t plane)
        Returns: See above.
        """
        def end_getHistogram(self, _r):
            return _M_omero.api.RawPixelsStore._op_getHistogram.end(self, _r)

        """
        Find the minimum and maximum pixel values for the specified channels by iterating over a full plane.
        In case of multi-z/t images only the 'middle' plane with index maxZ/2, respectively maxT/2 is taken into account.
        Note: This method can currently only handle non-pyramid images, otherwise an empty map will be returned.
        Arguments:
        channels -- the channels
        _ctx -- The request context for the invocation.
        Returns: See above.
        """
        def findMinMax(self, channels, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_findMinMax.invoke(self, ((channels, ), _ctx))

        """
        Find the minimum and maximum pixel values for the specified channels by iterating over a full plane.
        In case of multi-z/t images only the 'middle' plane with index maxZ/2, respectively maxT/2 is taken into account.
        Note: This method can currently only handle non-pyramid images, otherwise an empty map will be returned.
        Arguments:
        channels -- the channels
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_findMinMax(self, channels, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_findMinMax.begin(self, ((channels, ), _response, _ex, _sent, _ctx))

        """
        Find the minimum and maximum pixel values for the specified channels by iterating over a full plane.
        In case of multi-z/t images only the 'middle' plane with index maxZ/2, respectively maxT/2 is taken into account.
        Note: This method can currently only handle non-pyramid images, otherwise an empty map will be returned.
        Arguments:
        channels -- the channels
        Returns: See above.
        """
        def end_findMinMax(self, _r):
            return _M_omero.api.RawPixelsStore._op_findMinMax.end(self, _r)

        """
        Returns the byte width for the pixel store.
        Arguments:
        _ctx -- The request context for the invocation.
        Returns: See above.
        """
        def getByteWidth(self, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getByteWidth.invoke(self, ((), _ctx))

        """
        Returns the byte width for the pixel store.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getByteWidth(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_getByteWidth.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Returns the byte width for the pixel store.
        Arguments:
        Returns: See above.
        """
        def end_getByteWidth(self, _r):
            return _M_omero.api.RawPixelsStore._op_getByteWidth.end(self, _r)

        """
        Returns whether or not the pixel store has signed pixels.
        Arguments:
        _ctx -- The request context for the invocation.
        Returns: See above.
        """
        def isSigned(self, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_isSigned.invoke(self, ((), _ctx))

        """
        Returns whether or not the pixel store has signed pixels.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_isSigned(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_isSigned.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Returns whether or not the pixel store has signed pixels.
        Arguments:
        Returns: See above.
        """
        def end_isSigned(self, _r):
            return _M_omero.api.RawPixelsStore._op_isSigned.end(self, _r)

        """
        Returns whether or not the pixel buffer has floating point pixels.
        Arguments:
        _ctx -- The request context for the invocation.
        Returns: See above.
        """
        def isFloat(self, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_isFloat.invoke(self, ((), _ctx))

        """
        Returns whether or not the pixel buffer has floating point pixels.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_isFloat(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_isFloat.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Returns whether or not the pixel buffer has floating point pixels.
        Arguments:
        Returns: See above.
        """
        def end_isFloat(self, _r):
            return _M_omero.api.RawPixelsStore._op_isFloat.end(self, _r)

        """
        Calculates a SHA-1 message digest for the entire pixel store.
        Arguments:
        _ctx -- The request context for the invocation.
        Returns: byte array containing the message digest.
        """
        def calculateMessageDigest(self, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_calculateMessageDigest.invoke(self, ((), _ctx))

        """
        Calculates a SHA-1 message digest for the entire pixel store.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_calculateMessageDigest(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_calculateMessageDigest.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Calculates a SHA-1 message digest for the entire pixel store.
        Arguments:
        Returns: byte array containing the message digest.
        """
        def end_calculateMessageDigest(self, _r):
            return _M_omero.api.RawPixelsStore._op_calculateMessageDigest.end(self, _r)

        """
        Save the current state of the pixels, updating the SHA1. This should
        only be called AFTER all data is successfully set. Future invocations
        of set methods may be disallowed. This read-only status will allow
        background processing (generation of thumbnails, compression, etc.)
        to begin. More information under RawPixelsStore.
        A null instance will be returned if no save was performed.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def save(self, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_save.invoke(self, ((), _ctx))

        """
        Save the current state of the pixels, updating the SHA1. This should
        only be called AFTER all data is successfully set. Future invocations
        of set methods may be disallowed. This read-only status will allow
        background processing (generation of thumbnails, compression, etc.)
        to begin. More information under RawPixelsStore.
        A null instance will be returned if no save was performed.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_save(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.RawPixelsStore._op_save.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Save the current state of the pixels, updating the SHA1. This should
        only be called AFTER all data is successfully set. Future invocations
        of set methods may be disallowed. This read-only status will allow
        background processing (generation of thumbnails, compression, etc.)
        to begin. More information under RawPixelsStore.
        A null instance will be returned if no save was performed.
        Arguments:
        """
        def end_save(self, _r):
            return _M_omero.api.RawPixelsStore._op_save.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.api.RawPixelsStorePrx.ice_checkedCast(proxy, '::omero::api::RawPixelsStore', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.api.RawPixelsStorePrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::api::RawPixelsStore'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.api._t_RawPixelsStorePrx = IcePy.defineProxy('::omero::api::RawPixelsStore', RawPixelsStorePrx)

    _M_omero.api._t_RawPixelsStore = IcePy.defineClass('::omero::api::RawPixelsStore', RawPixelsStore, -1, (), True, False, None, (_M_omero.api._t_PyramidService,), ())
    RawPixelsStore._ice_type = _M_omero.api._t_RawPixelsStore

    RawPixelsStore._op_setPixelsId = IcePy.Operation('setPixelsId', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), IcePy._t_bool, False, 0)), (), None, (_M_omero._t_ServerError,))
    RawPixelsStore._op_getPixelsId = IcePy.Operation('getPixelsId', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), IcePy._t_long, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_getPixelsPath = IcePy.Operation('getPixelsPath', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), IcePy._t_string, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_prepare = IcePy.Operation('prepare', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero.sys._t_LongList, False, 0),), (), None, (_M_omero._t_ServerError,))
    RawPixelsStore._op_getPlaneSize = IcePy.Operation('getPlaneSize', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), IcePy._t_long, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_getRowSize = IcePy.Operation('getRowSize', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), IcePy._t_int, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_getStackSize = IcePy.Operation('getStackSize', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), IcePy._t_long, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_getTimepointSize = IcePy.Operation('getTimepointSize', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), IcePy._t_long, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_getTotalSize = IcePy.Operation('getTotalSize', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), IcePy._t_long, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_getRowOffset = IcePy.Operation('getRowOffset', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0)), (), ((), IcePy._t_long, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_getPlaneOffset = IcePy.Operation('getPlaneOffset', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0)), (), ((), IcePy._t_long, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_getStackOffset = IcePy.Operation('getStackOffset', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0)), (), ((), IcePy._t_long, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_getTimepointOffset = IcePy.Operation('getTimepointOffset', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_int, False, 0),), (), ((), IcePy._t_long, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_getTile = IcePy.Operation('getTile', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0)), (), ((), _M_Ice._t_ByteSeq, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_getHypercube = IcePy.Operation('getHypercube', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero.sys._t_IntList, False, 0), ((), _M_omero.sys._t_IntList, False, 0), ((), _M_omero.sys._t_IntList, False, 0)), (), ((), _M_Ice._t_ByteSeq, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_getRegion = IcePy.Operation('getRegion', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_int, False, 0), ((), IcePy._t_long, False, 0)), (), ((), _M_Ice._t_ByteSeq, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_getRow = IcePy.Operation('getRow', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0)), (), ((), _M_Ice._t_ByteSeq, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_getCol = IcePy.Operation('getCol', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0)), (), ((), _M_Ice._t_ByteSeq, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_getPlane = IcePy.Operation('getPlane', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0)), (), ((), _M_Ice._t_ByteSeq, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_getPlaneRegion = IcePy.Operation('getPlaneRegion', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0)), (), ((), _M_Ice._t_ByteSeq, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_getStack = IcePy.Operation('getStack', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0)), (), ((), _M_Ice._t_ByteSeq, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_getTimepoint = IcePy.Operation('getTimepoint', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_int, False, 0),), (), ((), _M_Ice._t_ByteSeq, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_setTile = IcePy.Operation('setTile', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_Ice._t_ByteSeq, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0)), (), None, (_M_omero._t_ServerError,))
    RawPixelsStore._op_setRegion = IcePy.Operation('setRegion', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_int, False, 0), ((), IcePy._t_long, False, 0), ((), _M_Ice._t_ByteSeq, False, 0)), (), None, (_M_omero._t_ServerError,))
    RawPixelsStore._op_setRow = IcePy.Operation('setRow', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_Ice._t_ByteSeq, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0)), (), None, (_M_omero._t_ServerError,))
    RawPixelsStore._op_setPlane = IcePy.Operation('setPlane', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_Ice._t_ByteSeq, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0)), (), None, (_M_omero._t_ServerError,))
    RawPixelsStore._op_setStack = IcePy.Operation('setStack', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_Ice._t_ByteSeq, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0)), (), None, (_M_omero._t_ServerError,))
    RawPixelsStore._op_setTimepoint = IcePy.Operation('setTimepoint', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_Ice._t_ByteSeq, False, 0), ((), IcePy._t_int, False, 0)), (), None, (_M_omero._t_ServerError,))
    RawPixelsStore._op_getHistogram = IcePy.Operation('getHistogram', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero.api._t_IntegerArray, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_bool, False, 0), ((), _M_omero.romio._t_PlaneDef, False, 0)), (), ((), _M_omero.api._t_IntegerIntegerArrayMap, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_findMinMax = IcePy.Operation('findMinMax', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero.api._t_IntegerArray, False, 0),), (), ((), _M_omero.api._t_IntegerDoubleArrayMap, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_getByteWidth = IcePy.Operation('getByteWidth', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), IcePy._t_int, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_isSigned = IcePy.Operation('isSigned', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), IcePy._t_bool, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_isFloat = IcePy.Operation('isFloat', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), IcePy._t_bool, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_calculateMessageDigest = IcePy.Operation('calculateMessageDigest', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), _M_Ice._t_ByteSeq, False, 0), (_M_omero._t_ServerError,))
    RawPixelsStore._op_save = IcePy.Operation('save', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), _M_omero.model._t_Pixels, False, 0), (_M_omero._t_ServerError,))

    _M_omero.api.RawPixelsStore = RawPixelsStore
    del RawPixelsStore

    _M_omero.api.RawPixelsStorePrx = RawPixelsStorePrx
    del RawPixelsStorePrx

# End of module omero.api

__name__ = 'omero'

# End of module omero



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# 
# Copyright 2015 Glencoe Software, Inc. All rights reserved.
# Use is subject to license terms supplied in LICENSE.txt

"""
Metadata plugin
"""

from __future__ import division

from builtins import str
from future.utils import native_str
from builtins import range
from past.utils import old_div
from builtins import object
import logging
import mimetypes
import os
import re
import sys
import warnings

import omero
from omero.cli import BaseControl
from omero.cli import CLI
from omero.cli import ProxyStringType
from omero.constants import namespaces
from omero.gateway import BlitzGateway
from omero.util import populate_metadata, populate_roi, pydict_text_io
from omero.util.metadata_utils import NSBULKANNOTATIONSCONFIG
from omero.util.metadata_utils import NSBULKANNOTATIONSRAW
from omero.grid import LongColumn
from omero.model.enums import UnitsLength

HELP = """Metadata utilities

Provides access to and editing of the metadata which
is typically shown in the right-hand panel of the
GUI clients.
"""

DEPRECATION_MESSAGE = (
    "This plugin is deprecated as of OMERO 5.4.8. Use the plugin"
    " available from https://pypi.org/project/omero-metadata/"
    " instead.")

ANNOTATION_TYPES = [t for t in dir(omero.model)
                    if re.match(r'[A-Za-z0-9]+Annotation$', t)]


def guess_mimetype(filename):
    mt = mimetypes.guess_type(filename, strict=False)[0]
    if not mt and os.path.splitext(filename) in ('yml', 'yaml'):
        mt = "application/x-yaml"
    if not mt:
        mt = "application/octet-stream"
    return mt


class ObjectLoadException(Exception):
    """
    Raised when a requested object could not be loaded
    """
    def __init__(self, klass, oid):
        self.klass = klass
        self.oid = oid
        super(ObjectLoadException, self).__init__(
            "Failed to get object %s:%s" % (klass, oid))


class Metadata(object):
    """
    A general helper object which providers higher-level
    accessors and mutators for a particular object.
    """

    def __init__(self, obj_wrapper):
        """
        Takes an initialized omero.gateway Wrapper object.
        """
        assert obj_wrapper
        self.obj_wrapper = obj_wrapper

    def get_type(self):
        t = self.obj_wrapper._obj.ice_staticId()
        return t.split("::")[-1]

    def get_id(self):
        return self.obj_wrapper.getId()

    def get_name(self):
        otype = self.get_type()
        oid = self.get_id()
        return "%s:%s" % (otype, oid)

    def get_parent(self):
        return self.wrap(self.obj_wrapper.getParent())

    def get_parents(self):
        return self.wrap(self.obj_wrapper.listParents())

    def get_roi_count(self):
        return self.obj_wrapper.getROICount()

    def get_original(self):
        return self.obj_wrapper.loadOriginalMetadata()

    def get_bulkanns(self):
        return self.get_allanns(namespaces.NSBULKANNOTATIONS)

    def get_measures(self):
        return self.get_allanns(namespaces.NSMEASUREMENT)

    def get_allanns(self, ns=None, anntype=None):
        # Sooner or later we're going to end up with an object that has 1000s
        # of annotations, and since listAnnotations() returns a generator we
        # might as well do the same
        for a in self.obj_wrapper.listAnnotations(ns):
            aw = self.wrap(a)
            if anntype and aw.get_type() != anntype:
                continue
            yield aw

    def wrap(self, obj):
        if obj is None:
            return None
        try:
            return [self.__class__(o) for o in obj]
        except TypeError:
            return self.__class__(obj)

    def __getattr__(self, name):
        return getattr(self.obj_wrapper, name)

    def __str__(self):
        return "<Metadata%s>" % self.obj_wrapper


class MetadataControl(BaseControl):

    POPULATE_CONTEXTS = (
        ("csv", populate_metadata.ParsingContext),
        ("bulkmap", populate_metadata.BulkToMapAnnotationContext),
        ("deletemap", populate_metadata.DeleteMapAnnotationContext),
    )

    def _configure(self, parser):
        parser.add_login_arguments()

        sub = parser.sub()
        summary = parser.add(sub, self.summary)
        original = parser.add(sub, self.original)
        measures = parser.add(sub, self.measures)
        bulkanns = parser.add(sub, self.bulkanns)
        mapanns = parser.add(sub, self.mapanns)
        allanns = parser.add(sub, self.allanns)
        parser.add(sub, self.testtables)
        rois = parser.add(sub, self.rois)
        populate = parser.add(sub, self.populate)
        populateroi = parser.add(sub, self.populateroi)
        pixelsize = parser.add(sub, self.pixelsize)

        populate.add_argument("--batch",
                              type=int,
                              default=1000,
                              help="Number of objects to process at once")
        self._add_wait(populate)

        for x in (summary, original, bulkanns, measures, mapanns, allanns,
                  rois, populate, populateroi, pixelsize):
            x.add_argument("obj",
                           type=ProxyStringType(),
                           help="Object in Class:ID format")

        for x in (bulkanns, measures, mapanns, allanns,
                  rois, populate, populateroi):
            x.add_argument("--report", action="store_true", help=(
                "Show additional information"))

        for x in (rois, populate, populateroi):
            dry_or_not = x.add_mutually_exclusive_group()
            dry_or_not.add_argument("-n", "--dry-run", action="store_true")
            dry_or_not.add_argument("-f", "--force", action="store_false",
                                    dest="dry_run")

        for x in (bulkanns, measures, mapanns, allanns):
            x.add_argument(
                "--parents", action="store_true",
                help="Also search parents for annotations")

        for x in (mapanns, allanns):
            x.add_argument(
                "--ns", default=None, help="Restrict to this namespace")
            x.add_argument(
                "--nsre",
                help="Restrict to this namespace (regular expression)")

        rois.add_argument(
            "--delete", action="store_true", help="Delete all ROIs")

        populate.add_argument(
            "--context", default=self.POPULATE_CONTEXTS[0][0],
            choices=[a[0] for a in self.POPULATE_CONTEXTS])

        datafile = populate.add_mutually_exclusive_group()
        datafile.add_argument("--file", help="Input file")
        datafile.add_argument(
            "--fileid", type=int, help="Input OriginalFile ID")

        cfgfile = populate.add_mutually_exclusive_group()
        cfgfile.add_argument("--cfg", help="YAML configuration file")
        cfgfile.add_argument(
            "--cfgid", type=int, help="YAML configuration OriginalFile ID")

        populate.add_argument("--attach", action="store_true", help=(
            "Upload input or configuration files and attach to parent object"))

        populate.add_argument("--localcfg", help=(
            "Local configuration file or a JSON object string"))

        populateroi.add_argument(
            "--measurement", type=int, default=None,
            help="Index of the measurement to populate. By default, all")

        pixelsize.add_argument(
            "--x", type=float, default=None, help="Physical pixel size X")
        pixelsize.add_argument(
            "--y", type=float, default=None, help="Physical pixel size Y")
        pixelsize.add_argument(
            "--z", type=float, default=None, help="Physical pixel size Z")
        pixelsize.add_argument(
            "--unit", default="micrometer",
            help="Unit (nanometer, micrometer, etc.) (default: micrometer)")

    def _clientconn(self, args):
        client = self.ctx.conn(args)
        conn = BlitzGateway(client_obj=client)
        return client, conn

    def _load(self, args, die_on_failure=True):
        # In most cases we want to die immediately if an object can't be
        # loaded. To raise an exception instead pass die_on_failure=False
        # and catch ObjectLoadException
        client, conn = self._clientconn(args)
        conn.SERVICE_OPTS.setOmeroGroup('-1')
        klass = args.obj.ice_staticId().split("::")[-1]
        oid = args.obj.id.val
        wrapper = conn.getObject(klass, oid)
        if not wrapper:
            e = ObjectLoadException(klass, oid)
            if die_on_failure:
                self.ctx.die(100, str(e))
            raise e
        return Metadata(wrapper)

    def _format_ann(self, md, obj, indent=None):
        "Format an annotation as a string, optionally pretty-printed"
        if indent is None:
            s = obj.get_name()
        else:
            s = "%s%s" % ("  " * indent, obj.get_name())
            pre = "\n%s" % ("  " * (indent + 1))
            s += "%sns: %s" % (pre, obj.getNs())
            s += "%sdescription: %s" % (pre, obj.getDescription())
            s += "%sdate: %s" % (pre, obj.getDate().isoformat())

            if obj.get_type() == 'FileAnnotation':
                f = md.wrap(obj.getFile())
                s += "%sfile: %s" % (pre, f.get_name())
                pre = "\n%s" % ("  " * (indent + 2))
                s += "%sname: %s" % (pre, f.getName())
                s += "%ssize: %s" % (pre, f.getSize())

            elif obj.get_type() == 'MapAnnotation':
                ma = obj.getValue()
                s += "%svalue:" % pre
                pre = "\n%s" % ("  " * (indent + 2))
                for k, v in ma:
                    s += "%s%s=%s" % (pre, k, v)

            else:
                v = obj.getValue()
                s += "%svalue: %s" % (pre, v)

        return s

    # READ METHODS

    def summary(self, args):
        "Provide a general summary of available metadata"
        self.ctx.err(DEPRECATION_MESSAGE, DeprecationWarning)
        md = self._load(args)
        name = md.get_name()
        line = "-" * len(name)
        self.ctx.out(name)
        self.ctx.out(line)
        try:
            self.ctx.out("Name: %s" % md.name)
        except AttributeError:
            pass
        try:
            self.ctx.out("Roi count: %s" % md.get_roi_count())
        except AttributeError:
            pass
        self.ctx.out("Bulk annotations: %d" %
                     sum(1 for a in md.get_bulkanns()))
        self.ctx.out("Measurement tables: %d" %
                     sum(1 for a in md.get_measures()))
        try:
            parent = md.get_parent().get_name()
            self.ctx.out("Parent: %s" % parent)
            otherparents = [p.get_name() for p in md.get_parents()]
            otherparents = [p for p in otherparents if p != parent]
            if otherparents:
                self.ctx.out("Other Parents: %s" % ",".join(otherparents))
        except AttributeError:
            pass
        try:
            source, global_om, series_om = md.get_original()
            self.ctx.out("Source metadata: %s" % bool(source))
            self.ctx.out("Global metadata: %s" % bool(global_om))
            self.ctx.out("Series metadata: %s" % bool(series_om))
        except AttributeError:
            pass

        counts = {}
        anns = md.get_allanns()
        for a in anns:
            try:
                counts[a.get_type()] += 1
            except KeyError:
                counts[a.get_type()] = 1
        for t in sorted(ANNOTATION_TYPES):
            if counts.get(t):
                self.ctx.out("%ss: %s" % (t, counts[t]))

    def original(self, args):
        "Print the original metadata in ini format"
        self.ctx.err(DEPRECATION_MESSAGE, DeprecationWarning)
        md = self._load(args)
        try:
            source, global_om, series_om = md.get_original()
        except AttributeError:
            self.ctx.die(100, 'Failed to get original metadata for %s' %
                         md.get_name())

        om = (("Global", global_om),
              ("Series", series_om))

        for name, tuples in om:
            # Matches the OMERO4 original_metadata.txt format
            self.ctx.out("[%sMetadata]" % name)
            for k, v in tuples:
                self.ctx.out("%s=%s" % (k, v))

    def _output_ann(self, mdobj, func, parents, indent):
        try:
            # Dereference the generator here so that we can catch
            # NotImplementedError
            anns = list(func(mdobj))
        except NotImplementedError:
            self.ctx.err('WARNING: Failed to get annotations for %s' %
                         mdobj.get_name())
            anns = []
        if indent is not None:
            self.ctx.out("%s%s" % (
                '  ' * indent, mdobj.get_name()))
            indent += 1
        for a in anns:
            self.ctx.out(self._format_ann(mdobj, a, indent))
        if parents:
            for p in mdobj.get_parents():
                self._output_ann(p, func, parents, indent)

    def bulkanns(self, args):
        ("Provide a list of the NSBULKANNOTATION tables linked "
         "to the given object")
        self.ctx.err(DEPRECATION_MESSAGE, DeprecationWarning)
        md = self._load(args)
        indent = None
        if args.report:
            indent = 0
        self._output_ann(
            md, lambda md: md.get_bulkanns(), args.parents, indent)

    def measures(self, args):
        ("Provide a list of the NSMEASUREMENT tables linked "
         "to the given object")
        self.ctx.err(DEPRECATION_MESSAGE, DeprecationWarning)
        md = self._load(args)
        indent = None
        if args.report:
            indent = 0
        self._output_ann(
            md, lambda md: md.get_measures(), args.parents, indent)

    def mapanns(self, args):
        "Provide a list of all MapAnnotations linked to the given object"
        def get_anns(md):
            for a in md.get_allanns(args.ns, 'MapAnnotation'):
                if args.nsre and not re.match(args.nsre, a.get_ns()):
                    continue
                yield a
        self.ctx.err(DEPRECATION_MESSAGE, DeprecationWarning)
        md = self._load(args)
        indent = None
        if args.report:
            indent = 0
        self._output_ann(md, get_anns, args.parents, indent)

    def allanns(self, args):
        "Provide a list of all annotations linked to the given object"
        def get_anns(md):
            for a in md.get_allanns(args.ns):
                if args.nsre and not re.match(args.nsre, a.get_ns()):
                    continue
                yield a
        self.ctx.err(DEPRECATION_MESSAGE, DeprecationWarning)
        md = self._load(args)
        indent = None
        if args.report:
            indent = 0
        self._output_ann(md, get_anns, args.parents, indent)

    def testtables(self, args):
        "Tests whether tables can be created and initialized"
        self.ctx.err(DEPRECATION_MESSAGE, DeprecationWarning)
        client, conn = self._clientconn(args)

        sf = client.getSession()
        sr = sf.sharedResources()
        table = sr.newTable(1, 'testtables')
        if table is None:
            self.ctx.die(100, "Failed to create Table")

        # If we have a table...
        initialized = False
        try:
            table.initialize([LongColumn('ID', '', [])])
            initialized = True
        except:
            pass
        finally:
            table.close()

        try:
            orig_file = table.getOriginalFile()
            conn.deleteObject(orig_file)
        except:
            # Anything else to do here?
            pass

        if not initialized:
            self.ctx.die(100, "Failed to initialize Table")

    # WRITE

    def populate(self, args):
        "Add metadata (bulk-annotations) to an object"
        self.ctx.err(DEPRECATION_MESSAGE, DeprecationWarning)
        md = self._load(args)
        client, conn = self._clientconn(args)
        # TODO: Configure logging properly
        if args.report:
            populate_metadata.log.setLevel(logging.DEBUG)
        else:
            populate_metadata.log.setLevel(logging.INFO)

        context_class = dict(self.POPULATE_CONTEXTS)[args.context]

        if args.localcfg:
            localcfg = pydict_text_io.load(
                args.localcfg, session=client.getSession())
        else:
            localcfg = {}

        fileid = args.fileid
        cfgid = args.cfgid

        if args.attach and not args.dry_run:
            if args.file:
                fileann = conn.createFileAnnfromLocalFile(
                    args.file, mimetype=guess_mimetype(args.file),
                    ns=NSBULKANNOTATIONSRAW)
                fileid = fileann.getFile().getId()
                md.linkAnnotation(fileann)

            if args.cfg:
                cfgann = conn.createFileAnnfromLocalFile(
                    args.cfg, mimetype=guess_mimetype(args.cfg),
                    ns=NSBULKANNOTATIONSCONFIG)
                cfgid = cfgann.getFile().getId()
                md.linkAnnotation(cfgann)

        # Note some contexts only support a subset of these args
        ctx = context_class(client, args.obj, file=args.file, fileid=fileid,
                            cfg=args.cfg, cfgid=cfgid, attach=args.attach,
                            options=localcfg)
        ctx.parse()
        if not args.dry_run:
            wait = args.wait
            if not wait:
                loops = 0
                ms = 0
            else:
                ms = 5000
                loops = int(old_div((wait * 1000), ms)) + 1
            ctx.write_to_omero(batch_size=args.batch, loops=loops, ms=ms)

    def rois(self, args):
        "Manage ROIs"
        self.ctx.err(DEPRECATION_MESSAGE, DeprecationWarning)
        md = self._load(args)
        if args.delete:
            graphspec = "/%s/Roi:%d" % (md.get_type(), md.get_id())
            cmd = ["delete", graphspec]
            if args.report:
                cmd += ["--report"]
            if args.dry_run:
                cmd += ["--dry-run"]
            return self.ctx.invoke(cmd)

        client = self.ctx.conn(args)
        params = omero.sys.ParametersI()
        params.addId(md.get_id())
        if md.get_type() == "Screen":
            q = """SELECT r.id FROM Roi r, WellSample ws, Plate p,
                   ScreenPlateLink spl WHERE
                   spl.child=p AND r.image=ws.image AND ws.well.plate=p AND
                   spl.parent.id=:id"""
        elif md.get_type() == "Plate":
            q = """SELECT r.id FROM Roi r, WellSample ws WHERE
                   r.image=ws.image AND ws.well.plate.id=:id"""
        elif md.get_type() == "PlateAcquisition":
            q = """SELECT r.id FROM Roi r, WellSample ws WHERE
                   r.image=ws.image AND ws.plateAcquisition.id=:id"""
        elif md.get_type() == "Well":
            q = """SELECT r.id FROM Roi r, WellSample ws WHERE
                   r.image=ws.image AND ws.well.id=:id"""
        elif md.get_type() == "Project":
            q = """SELECT r.id FROM Roi r, DatasetImageLink dil,
                   ProjectDatasetLink pdl WHERE dil.child=r.image AND
                   dil.parent=pdl.child AND pdl.parent.id=:id"""
        elif md.get_type() == "Dataset":
            q = """SELECT r.id FROM Roi r, DatasetImageLink dil WHERE
                   dil.child=r.image AND dil.parent.id=:id"""
        elif md.get_type() == "Image":
            q = """SELECT r.id FROM Roi r WHERE r.image.id=:id"""
        else:
            raise Exception("Not implemented for type %s" % md.get_type())
        roiids = client.getSession().getQueryService().projection(q, params)
        roiids = [r[0].val for r in roiids]
        if roiids:
            self.ctx.out('\n'.join('Roi:%d' % rid for rid in roiids))

    def populateroi(self, args):
        "Add ROIs to an object"
        self.ctx.err(DEPRECATION_MESSAGE, DeprecationWarning)
        md = self._load(args)
        client = self.ctx.conn(args)
        # TODO: Configure logging properly
        if args.report:
            populate_roi.log.setLevel(logging.DEBUG)
        else:
            populate_roi.log.setLevel(logging.INFO)
        factory = populate_roi.PlateAnalysisCtxFactory(client.sf)
        ctx = factory.get_analysis_ctx(md.get_id())
        count = ctx.get_measurement_count()
        if not count:
            self.ctx.die(100, "No measurements found")
        if args.measurement is not None and args.measurement >= count:
            self.ctx.die(
                100, "Invalid measurement index: %d" % args.measurement)
        for i in range(count):
            if args.dry_run:
                self.ctx.out(
                    "Measurement %d has %s result files." % (
                        i, ctx.get_result_file_count(i)))
            else:
                if args.measurement is not None:
                    if args.measurement != i:
                        continue
                meas = ctx.get_measurement_ctx(i)
                meas.parse_and_populate()

    def pixelsize(self, args):
        "Set physical pixel size"
        self.ctx.err(DEPRECATION_MESSAGE, DeprecationWarning)
        if not args.x and not args.y and not args.z:
            self.ctx.die(100, "No pixel sizes specified.")

        unit = getattr(UnitsLength, args.unit.upper())
        if not unit:
            self.ctx.die(100, "%s is not recognized as valid unit."
                              % args.unit)

        md = self._load(args)
        client, conn = self._clientconn(args)

        if md.get_type() == "Screen":
            q = """SELECT pix FROM Pixels pix, WellSample ws, Plate p,
                   ScreenPlateLink spl WHERE
                   spl.child=p AND pix.image=ws.image AND ws.well.plate=p AND
                   spl.parent.id=:id"""
        elif md.get_type() == "Plate":
            q = """SELECT pix FROM Pixels pix, WellSample ws WHERE
                   pix.image=ws.image AND ws.well.plate.id=:id"""
        elif md.get_type() == "PlateAcquisition":
            q = """SELECT pix FROM Pixels pix, WellSample ws WHERE
                   pix.image=ws.image AND ws.plateAcquisition.id=:id"""
        elif md.get_type() == "Well":
            q = """SELECT pix FROM Pixels pix, WellSample ws WHERE
                   pix.image=ws.image AND ws.well.id=:id"""
        elif md.get_type() == "Project":
            q = """SELECT pix FROM Pixels pix, DatasetImageLink dil,
                   ProjectDatasetLink pdl WHERE dil.child=pix.image AND
                   dil.parent=pdl.child AND pdl.parent.id=:id"""
        elif md.get_type() == "Dataset":
            q = """SELECT pix FROM Pixels pix, DatasetImageLink dil WHERE
                   dil.child=pix.image AND dil.parent.id=:id"""
        elif md.get_type() == "Image":
            q = """SELECT pix FROM Pixels pix WHERE pix.image.id=:id"""
        else:
            raise Exception("Not implemented for type %s" % md.get_type())

        ctx = {'omero.group': '-1'}

        params = omero.sys.ParametersI()
        params.addId(md.get_id())
        pixels = conn.getQueryService().findAllByQuery(q, params, ctx)

        if not pixels:
            self.ctx.die(100, "Failed to get Pixel object(s)")

        for pixel in pixels:
            if args.x:
                pixel.setPhysicalSizeX(omero.model.LengthI(args.x, unit))
            if args.y:
                pixel.setPhysicalSizeY(omero.model.LengthI(args.y, unit))
            if args.z:
                pixel.setPhysicalSizeZ(omero.model.LengthI(args.z, unit))

        groupId = pixels[0].getDetails().getGroup().getId().getValue()
        ctx = {'omero.group': native_str(groupId)}
        conn.getUpdateService().saveArray(pixels, ctx)


try:
    if "OMERO_DEV_PLUGINS" in os.environ:
        warnings.warn(DEPRECATION_MESSAGE, DeprecationWarning)
        register("metadata", MetadataControl, HELP)
except NameError:
    if __name__ == "__main__":
        cli = CLI()
        cli.register("metadata", MetadataControl, HELP)
        cli.invoke(sys.argv[1:])
